﻿'***************************************************************************************************
'     汎用メッセージボックス共通部(システム共通)            modMessageBox2(Module)
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 13/03/17(1.0.4.0)新規作成
' 15/02/05(1.0.5.0)clsMessageBox2よりファイル分離
' 17/10/29(1.0.8.0)表示サイズ調整機能を追加(dlgMessageBox2D用)
'***************************************************************************************************
Friend Module modMessageBox2
    '===============================================================================================
    Friend Const g_cnsFontFamily As String = "ＭＳ ゴシック"

    '***********************************************************************************************
    ' ■■■ 共通サブ処理(clsMessageBox2用) ■■■
    '***********************************************************************************************
    '* 処理名　：FP_CanUseOwnerForm
    '* 機能　　：FormStartPosition.CenterParent使用可否判定
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：判定結果(Boolean)
    '* 引数　　：Arg1 = OwnerForm(Object)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2013年02月24日
    '* 作成者　：井上　治
    '* 更新日　：2013年03月07日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Function FP_CanUseOwnerForm(ByVal objOwner As Object) As Boolean
        '-------------------------------------------------------------------------------------------
        If objOwner Is Nothing Then
            Return False
        End If
        ' 引数のフォームが参照可能かを確認
        Dim objOwner2 As Form = Nothing
        Try
            objOwner2 = objOwner
            If ((objOwner2.Left <> 0) OrElse _
                (objOwner2.Top <> 0) OrElse _
                (objOwner2.Width > 10) OrElse _
                (objOwner2.Height > 10) OrElse _
                (objOwner2.WindowState = FormWindowState.Maximized)) Then
                FP_CanUseOwnerForm = True
            Else
                ' 隠しフォームなどが起点となる場合はCenterScreenとなるようにする
                FP_CanUseOwnerForm = False
            End If
        Catch ex As Exception
            ' 参照失敗
            Return False
        End Try
    End Function

    '***********************************************************************************************
    ' ■■■ 共通サブ処理(dlgMessageBox2,dlgMessageBox2D共用) ■■■
    '***********************************************************************************************
    '* 処理名　：GP_SelectButtons
    '* 機能　　：ボタンの指定
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = MessageBoxフォーム(Object)
    '* 　　　　　Arg2 = ボタン指定(MessageBoxButtons)            ※Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2013年03月17日
    '* 作成者　：井上　治
    '* 更新日　：2013年06月06日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_SelectButtons( _
        ByRef objMessageBoxForm As Object, _
        Optional ByVal intButtons As MessageBoxButtons = MessageBoxButtons.OK)
        '-------------------------------------------------------------------------------------------
        With objMessageBoxForm
            Select Case intButtons
                Case MessageBoxButtons.OKCancel
                    ' OK､キャンセル
                    Call GP_SetButton(.Button1, True, "OK", DialogResult.OK)
                    Call GP_SetButton(.Button2, True, "キャンセル", DialogResult.Cancel)
                    Call GP_SetButton(.Button3, False)
                    .ControlBox = True
                    .CancelButton = .Button2
                Case MessageBoxButtons.YesNo
                    ' はい､いいえ
                    Call GP_SetButton(.Button1, True, "はい(&Y)", DialogResult.Yes)
                    Call GP_SetButton(.Button2, True, "いいえ(&N)", DialogResult.No)
                    Call GP_SetButton(.Button3, False)
                    .ControlBox = False
                    .CancelButton = Nothing
                Case MessageBoxButtons.YesNoCancel
                    ' はい､いいえ､キャンセル
                    Call GP_SetButton(.Button1, True, "はい(&Y)", DialogResult.Yes)
                    Call GP_SetButton(.Button2, True, "いいえ(&N)", DialogResult.No)
                    Call GP_SetButton(.Button3, True, "キャンセル", DialogResult.Cancel)
                    .ControlBox = True
                    .CancelButton = .Button3
                Case MessageBoxButtons.RetryCancel
                    ' 再試行､キャンセル
                    Call GP_SetButton(.Button1, True, "再試行(&R)", DialogResult.Retry)
                    Call GP_SetButton(.Button2, True, "キャンセル", DialogResult.Cancel)
                    Call GP_SetButton(.Button3, False)
                    .ControlBox = True
                    .CancelButton = .Button2
                Case MessageBoxButtons.AbortRetryIgnore
                    ' 中止､再試行､無視
                    Call GP_SetButton(.Button1, True, "中止(&A)", DialogResult.Abort)
                    Call GP_SetButton(.Button2, True, "再試行(&R)", DialogResult.Retry)
                    Call GP_SetButton(.Button3, True, "無視(&I)", DialogResult.Ignore)
                    .ControlBox = False
                    .CancelButton = Nothing
                Case Else
                    ' OKボタンのみ
                    Call GP_SetButton(.Button1, True, "OK", DialogResult.OK)
                    Call GP_SetButton(.Button2, False)
                    Call GP_SetButton(.Button3, False)
                    .ControlBox = True
                    .CancelButton = .Button1
            End Select
            .AcceptButton = .Button1
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_SelectIcon
    '* 機能　　：アイコンの指定
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = MessageBoxフォーム(Object)
    '* 　　　　　Arg2 = アイコン指定(MessageBoxIcon)             ※Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2013年03月17日
    '* 作成者　：井上　治
    '* 更新日　：2013年03月17日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_SelectIcon( _
        ByRef objMessageBoxForm As Object, _
        Optional ByVal intIcon As MessageBoxIcon = MessageBoxIcon.None)
        '-------------------------------------------------------------------------------------------
        With objMessageBoxForm
            Select Case intIcon
                Case MessageBoxIcon.Error, MessageBoxIcon.Stop, MessageBoxIcon.Hand
                    ' Error
                    .IMG_ERROR.Visible = True
                    .IMG_EXCLAMATION.Visible = False
                    .IMG_INFORMATION.Visible = False
                    .IMG_QUESTION.Visible = False
                    .PNL_IMAGE.Visible = True
                    System.Media.SystemSounds.Hand.Play()
                Case MessageBoxIcon.Exclamation, MessageBoxIcon.Warning
                    ' Exclamation
                    .IMG_ERROR.Visible = False
                    .IMG_EXCLAMATION.Visible = True
                    .IMG_INFORMATION.Visible = False
                    .IMG_QUESTION.Visible = False
                    .PNL_IMAGE.Visible = True
                    System.Media.SystemSounds.Exclamation.Play()
                Case MessageBoxIcon.Information, MessageBoxIcon.Asterisk
                    ' Information
                    .IMG_ERROR.Visible = False
                    .IMG_EXCLAMATION.Visible = False
                    .IMG_INFORMATION.Visible = True
                    .IMG_QUESTION.Visible = False
                    .PNL_IMAGE.Visible = True
                    System.Media.SystemSounds.Asterisk.Play()
                Case MessageBoxIcon.Question
                    ' Quention
                    .IMG_ERROR.Visible = False
                    .IMG_EXCLAMATION.Visible = False
                    .IMG_INFORMATION.Visible = False
                    .IMG_QUESTION.Visible = True
                    .PNL_IMAGE.Visible = True
                    System.Media.SystemSounds.Question.Play()
                Case Else
                    ' None
                    .IMG_ERROR.Visible = False
                    .IMG_EXCLAMATION.Visible = False
                    .IMG_INFORMATION.Visible = False
                    .IMG_QUESTION.Visible = False
                    .PNL_IMAGE.Visible = False
            End Select
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_SelectDefault
    '* 機能　　：デフォルトの指定
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = MessageBoxフォーム(Object)
    '* 　　　　　Arg2 = デフォルトボタン指定(MessageBoxDefaultButton)※Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2013年03月17日
    '* 作成者　：井上　治
    '* 更新日　：2013年03月17日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_SelectDefault( _
        ByRef objMessageBoxForm As Object, _
        Optional ByVal objButton As MessageBoxDefaultButton = Nothing)
        '-------------------------------------------------------------------------------------------
        With objMessageBoxForm
            Select Case objButton
                Case MessageBoxDefaultButton.Button2
                    ' Button2がデフォルト
                    .AcceptButton = .Button2
                    .Button1.TabIndex = 2
                    .Button2.TabIndex = 0
                    .Button3.TabIndex = 1
                Case MessageBoxDefaultButton.Button3
                    ' Button3がデフォルト
                    .AcceptButton = .Button3
                    .Button1.TabIndex = 1
                    .Button2.TabIndex = 2
                    .Button3.TabIndex = 0
                Case Else
                    ' Button1がデフォルト
                    .AcceptButton = .Button1
                    .Button1.TabIndex = 0
                    .Button2.TabIndex = 1
                    .Button3.TabIndex = 2
            End Select
        End With
    End Sub

    '***********************************************************************************************
    ' ■■■ dlgMessageBox2D用サブ処理 ■■■
    '***********************************************************************************************
    '* 処理名　：GP_AdjustFormSize
    '* 機能　　：フォーム(テキストボックス)のサイズ調整
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 対象フォーム(Object)
    '* 　　　　　Arg2 = 対象テキストボックス(Object)
    '* 　　　　　Arg3 = 表示テキスト(String)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年10月29日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月29日
    '* 更新者　：井上　治
    '* 機能説明：改行付きテキストの最大幅に合わせてフォームサイズを調整、高さは行数で調整
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_AdjustFormSize(ByRef objForm As Form, _
                                 ByRef objTextBox As TextBox, _
                                 ByVal strText As String)
        '-------------------------------------------------------------------------------------------
        Dim intWidthF As Integer = objForm.Width                    ' フォームの幅
        Dim intWidthT As Integer = objTextBox.Width                 ' テキストボックスの幅
        Dim intLength As Integer = FP_GetLengthShowable(objTextBox) ' 現在の表示可能バイト数
        Dim sngHeight As Single = objTextBox.Font.GetHeight         ' フォントの高さ
        Dim intCntLine As Integer = 0                               ' 表示行数
        Dim intLenMax As Integer = FP_GetMaxLengthFromText(strText, intCntLine) ' 最大幅バイト数
        ' 改行がない時はサイズ変更しない
        If intCntLine <= 1 Then Exit Sub
        Dim intHeightDiff As Integer = objForm.Height - objTextBox.Height ' テキストボックス以外の高さ
        ' 最大バイト数表示可能なテキストボックスの幅を算出
        Dim intWidthT2 As Integer = intWidthT * intLenMax / intLength ' テキストボックスの幅
        ' フォームの幅を算出(スクロールバー分を適当に加算)
        Dim intWidth2 As Integer = intWidthF + intWidthT2 - intWidthT + 22 ' 修正後の幅
        ' フォームの高さを算出(テキストボックスの行間と上下マージンを適当に加算)
        Dim intHeight2 As Integer = sngHeight * intCntLine + intHeightDiff + 6 ' 修正後の高さ
        '-------------------------------------------------------------------------------------------
        ' フォームサイズの調整
        With objForm
            ' 幅を調整(最低幅・最大幅を確認)
            If intWidth2 < .MinimumSize.Width Then
                .Width = .MinimumSize.Width
            ElseIf intWidth2 > .MaximumSize.Width Then
                .Width = .MaximumSize.Width
            Else
                .Width = intWidth2
            End If
            ' 高さを調整(最低高・最大高を確認)
            If intHeight2 < .MinimumSize.Height Then
                .Height = .MinimumSize.Height
            ElseIf intHeight2 > .MaximumSize.Height Then
                .Height = .MaximumSize.Height
            Else
                .Height = intHeight2
            End If
        End With
    End Sub

    '***********************************************************************************************
    ' ■■■ 共通サブ処理(Private) ■■■
    '***********************************************************************************************
    '* 処理名　：GP_SetButton
    '* 機能　　：ボタンの指定(個別)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = ボタン(Button)
    '* 　　　　　Arg2 = 表示有無(Boolean)
    '* 　　　　　Arg3 = 表示テキスト(String)
    '* 　　　　　Arg4 = クリック時の戻り値(DialogResult)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2013年03月17日
    '* 作成者　：井上　治
    '* 更新日　：2013年03月17日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SetButton(ByRef objButton As Button, _
                             ByVal blnVisible As Boolean, _
                             Optional ByVal strText As String = "", _
                             Optional ByVal intReturnValue As DialogResult = DialogResult.OK)
        '-------------------------------------------------------------------------------------------
        With objButton
            .Text = strText
            .Visible = blnVisible
            .Tag = intReturnValue
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_GetLengthShowable
    '* 機能　　：テキストボックスの表示可能バイト数取得
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：表示可能バイト数(Integer)
    '* 引数　　：Arg1 = 対象テキストボックス(Object)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年10月29日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月29日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_GetLengthShowable(ByVal objTextBox As TextBox) As Integer
        '-------------------------------------------------------------------------------------------
        Dim objTextBox2 As New TextBox                              ' ダミーテキストボックス
        Dim cnsStrings As String = New String("a", 5000)            ' ダミーテキスト
        Dim intPos As Integer = 0                                   ' 文字位置INDEX
        With objTextBox2
            ' フォントを調整
            .Font = objTextBox.Font
            .Width = objTextBox.Width
            .Multiline = False
            .WordWrap = False
            Dim intWidth As Integer = .Width                        ' 表示幅
            .Text = cnsStrings
            Do While intPos <= .TextLength
                ' 文字の表示位置がテキストボックスの幅を超える文字位置を取得
                If .GetPositionFromCharIndex(intPos).X >= intWidth Then
                    ' 1文字戻して終了
                    intPos -= 1
                    Exit Do
                End If
                intPos += 1
            Loop
            .Text = String.Empty
        End With
        Return intPos
    End Function

    '***********************************************************************************************
    '* 処理名　：FP_GetMaxLengthFromText
    '* 機能　　：改行付きテキストの最大幅バイト数
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：最大幅バイト数(Integer)
    '* 引数　　：Arg1 = 表示テキスト(String)
    '* 　　　　　Arg2 = 改行数(Integer)                     ※Ref参照
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年10月29日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月29日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_GetMaxLengthFromText(ByVal strText As String, _
                                             ByRef intCntLine As Integer) As Integer
        '-------------------------------------------------------------------------------------------
        Dim intIx As Integer = 0                                    ' テーブルINDEX
        Dim intLength As Integer = 0                                ' バイト数
        Dim tblText() As String = strText.Split(ControlChars.CrLf)  ' 改行で分割したテキスト
        ' 1行の場合はLFコードで再分割
        If tblText.Length <= 1 Then
            tblText = strText.Split(ControlChars.Lf)
        End If
        intCntLine = tblText.Length
        ' テーブルを巡回
        Do While intIx < intCntLine
            Dim intLength2 As Integer = FP_GetByteLength(tblText(intIx)) ' バイト数(現在行)
            ' 最大幅か
            If intLength2 > intLength Then intLength = intLength2
            ' 次へ
            intIx += 1
        Loop
        Return intLength
    End Function

    '***********************************************************************************************
    '* 処理名　：FP_GetByteLength
    '* 機能　　：文字列バイト長取得(全半角勘案)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：文字列バイト長(Integer)
    '* 引数　　：Arg1 = 対象テキスト(String)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年10月29日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月29日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_GetByteLength(ByVal strTEXT As String) As Integer
        '-------------------------------------------------------------------------------------------
        Dim intPos As Integer = 0                                   ' 文字位置INDEX
        ' 1文字ずつ半角/全角を検査
        FP_GetByteLength = 0
        Do While intPos < strTEXT.Length
            Dim intChar As Integer = Asc(strTEXT.Substring(intPos, 1)) ' 文字コード
            If ((intChar >= 0) AndAlso (intChar <= 255)) Then
                FP_GetByteLength += 1                       ' 半角は1バイト
            Else
                FP_GetByteLength += 2                       ' 全角は2バイト
            End If
            ' 次へ
            intPos += 1
        Loop
    End Function

    '---------------------------------------<< End of Source >>-------------------------------------
End Module

